######## FOR-RESTOR Synthesis Phil Trans Paper Analysis ##########
######## Banin, Raine et al.                            ###############
######## 02 November 2023
######## Height non linear models for estimating Annual Growth Rates 
######## at reference heights

#' This code was run using R version 4.0.4 to produce the results presented in 
#' the manuscript. Here, the code has been updated to run on R version 4.3.0. 
#' This has resulted in some updates to the functions used that yield different 
#' results for a small number of data points. To exactly replicate the results 
#' used in the paper, please use R version 4.0.4.

#' This script is used to create the AGR outputs for analysis in lmes.
#' It first identifies which of the treatments (site/species/ treatment combinations) 
#' include the reference heights of 100cm, 200cm and/or 300cm in their range.

#' It loops through each treatment and fits linear, exponential and gompertz models 
#' for height data for treatments with 4+ censuses.
#' It fits just the linear and exponential models for height data for treatments with 3 censuses.
#' It identifies the optimal model using AIC.
#'
#' In some cases in these for loops, errors will be returned whilst the loop is running:
#' this is just showing instances when some of the models have not converged, 
#' it is not a problem in the execution of the code.

#' ############## NOTE: 
#' If ran in R 4.3, some for loop warnings will be returned about the specification
#' of a self start function. This is due to a change in the way the function is
#' specified which happened after R 4.0.4. The change in the specification of the
#' self start function that appears to affect the way the exponential and gompertz 
#' models are fitted. Presumably as a result of this change, in 6 out of the 392 
#' treatments fitted in this code return a different result when fit in R 4.3.  
#' 
#' As these 6 treatments appear in several of the final reference height datasets,
#' when the models are fit using R 4.3, there are:
#' - 4 different AGR values in the dataframe for the 100cm reference height, 
#' - 6 different AGR values in the dataframe for the 200cm reference height and 
#' - 5 different AGR values in the dataframe for the 300cm reference height, 
#'compared to the data presented in the manuscript which used R 4.0.4 for analysis.
#' ####################

#' The code then refits the optimal model and calculates the time at which the reference height
#' is reached based on the model fit. It then calculates the growth in cm for
#' the 1 year period around this reference height. 
#'
#' It also calculates the variance of this annual growth rate using the delta method.
#' This involves calculating the derivative of the model with respect of each parameter.

#' The r script 'height_nlm_functions.R' contains additional functions required in this script. 
#' Executing this script also requires the functions from the open-access Paine (2012) paper 
#' on fitting growth models to function:

#' Paine CET, Marthews TR, Vogt DR, Purves D, Rees M, Hector A, Turnbull LA. 
#' 2012 How to fit nonlinear plant growth models and calculate growth rates: 
#' An update for ecologists. Methods Ecol. Evol. 3, 245-256. 
#' (doi:10.1111/j.2041-210X.2011.00155.x)
#' 
#' Download the Supplementary material R script: 'mee3_155_sm_appendixs1.R'
#' Save this in your working directory as 'Paine_functions.R', and they are 
#' loaded in the script below

rm(list=ls())
library(lme4)
library(nlme)
library(stats)
library(glmmTMB)
library(tidyr)
library(dplyr)
library(scales)
library(data.table)
library(mvtnorm)

set.seed(42)

# load functions from Paine (2012): you will have to download these yourself
source("Paine_functions.R")

# load functions for calculating derivatives

source("height_nlm_functions.R")

#######################################################################
# Setting up the dataframe: 

# read in height data
# format: one row for each height census per treatment

df_h <- read.csv("df_height_Oct2023.csv")

# extract treatments with >=3 censuses:
df_h3p <- df_h[!is.na(df_h$height_cm),]
df_h3p <- df_h3p[order(df_h3p$duration_months),]
g <- as.data.frame(table(df_h3p$plot_id))
g <- droplevels(g$Var1[g$Freq>=3])
df_h3p <- df_h3p[df_h3p$plot_id %in% g,]


# creating dataframe with the height range per plot_id
df_refh <- df_h3p %>% group_by(plot_id) %>% dplyr::summarise(minimum_height=min(height_cm_v2),
                                                   maximum_height=max(height_cm_v2))
df_refh$cm100 <- NA
df_refh$cm200 <- NA
df_refh$cm300 <- NA

## calculating whether reference heights fall within treatment height range:
for(i in 1:nrow(df_refh)){
  p <- df_refh[i,]
  if(p$minimum_height< 100 & p$maximum_height > 100){
    df_refh$cm100[i] <- "100"
  } 

  if(p$minimum_height< 200 & p$maximum_height > 200){
    df_refh$cm200[i] <- "200"
  } 
  if(p$minimum_height< 300 & p$maximum_height > 300){
    df_refh$cm300[i] <- "300"
  } 
}

rm(p);  rm(g)

###############################################################################
## Extracting data with 4+ censuses and setting up output table:

df_h3p <- df_h3p[order(df_h3p$duration_months),]
g <- as.data.frame(table(df_h3p$plot_id))
g <- droplevels(g$Var1[g$Freq>3])
df_h4 <- df_h3p[df_h3p$plot_id %in% g,]

rm(g)
#creating a df "id" with just these plot_ids and UIC (site)
id <- df_h4 %>% group_by(plot_id, UIC) %>% dplyr::summarise(length=length(height_cm_v2))

# Creating a dataframe for output of model runs
# Has one row for each plot_id and a column for each model and its parameters
# no_census: the number of censuses carried out at the site
# h_0: initial height
# t_100, t_200, t_300: the time at which the model predicts the reference heights 
# will be reached
# AGR and SGR at the three reference heights


df = data.frame(matrix(vector(), nrow(id),20,
                       dimnames=list(c(), c("UIC", "plot_id","code", "no_census", "h_0", # plot_id info
                                            "exp_AIC", # exp model outputs
                                             "gomp_AIC", #gomp model outputs
                                            "lm_AIC", # lm model outputs
                                            "t_100", "t_200", "t_300", #height at time of each reference height
                                            "AGR_100", "AGR_200", "AGR_300", # annual growth rate for year around each ref height
                                            "AGR100_var", "AGR200_var", "AGR300_var",
                                            "SGR_100", "SGR_200", "SGR_300" # annual growth rate for year around each ref height
                                            #"SGR100_var", "SGR200_var", "SGR300_var" #variance of each ref height AGR
                                            ))), 
                stringsAsFactors=F)

df$plot_id <- id$plot_id
df$UIC <- id$UIC
df$no_census <- id$length
df$code <- 1:nrow(df)

# add in table to identify which reference heights are encompassed by each treatment
df <- left_join(df, df_refh[,c(1,4:6)], by="plot_id")
df <- df[!is.na(df$cm100)| !is.na(df$cm200) |!is.na(df$cm300),]

################################################################################
# TESTING MODELS ON 4+ OBSERVATIONS
# This set of for loops runs each of the model types (exponential, gompertz and lm)
# for each of the treatments
# It fits the model and extracts the AIC and some parameter values
# Saving this into the "df" dataframe (created above)

###################################################################################
### FITTING THE LINEAR MODEL:

for(i in 1:nrow(df)){
   print(i)

  # extract the treatment data from the full dataframe
  p <- df$plot_id[i]
  g <- df_h4[df_h4$plot_id == p,]
  g <- g[,c("height_cm_v2", "duration_months")]
  
  #set the starting height
  h_0 <- round(g$height_cm_v2[1], digits=2) 
  df$h_0[i] <- h_0
 
  #Linear model:
  mod_lm <- lm(height_cm_v2 ~ duration_months, data= g)
 
  # extracting coefficients
  df$lm_AIC[i] <- AIC(mod_lm)

  }


###############################################################################
## EXPONENTIAL MODEL:

for(i in 1:nrow(df)){
  print(i)
  
  # extract the treatment data from the full dataframe
  p <- df$plot_id[i]
  g <- df_h4[df_h4$plot_id == p,]
  g <- g[,c("height_cm_v2", "duration_months")]
  
  #set the starting height
  h_0 <- round(g$height_cm[1], digits=2) 
  
  # this is a function from Paine (2012) to get starting values
  tmp.exp  <- getInitial(object = height_cm_v2~SS.exp(duration_months, h_0, r), data = g)

  # add these variables into the dataframe: 
  r_s <- as.numeric(tmp.exp[2])
  h0_s <- as.numeric(tmp.exp[1])
  
  # fit the model
  fit.exp <- nls(height_cm_v2 ~ h0* exp(r*duration_months),
                  start = list(h0=h0_s, r = r_s),
                  data = g,
                 control = list(maxiter = 500))

  # extract AIC
  df$exp_AIC[i] <- AIC(fit.exp)
 }

# you get warnings about:
# selfStart initializing functions should have a final '...' argument since R 4.1.0
# but this is fine

################################################################################
# GOMPERTZ:

# Here the warnings in the for loop are turned off so the for loop doesn't 
# stop when one of the models doesn't converge.

for(i in 1:nrow(df)){
  tryCatch({
  print(i)

  # extract the treatment data from the full dataframe
  p <- df$plot_id[i]
  g <- df_h4[df_h4$plot_id == p,]
  g <- g[,c("height_cm_v2", "duration_months")]
  
  # get start values
  tmp.gomp <- getInitial(height_cm_v2 ~ SSgompertz(duration_months, Asym, b2, b3), 
                         data = g)
  fit.gomp_trial <- gnls(height_cm_v2 ~ SSgompertz(duration_months, Asym, b2, b3), 
                         data = g, weights= varExp(form = ~ fitted(.)))
  out.gomp <- output.gomp.gnls(fit.gomp_trial, g$duration_months)
 
  k_s <- as.numeric(out.gomp$params[1])
  r_s <- as.numeric(out.gomp$params[2])
  h0_s <- as.numeric(out.gomp$params[3])
  
  fit.gomp <- nls(height_cm_v2 ~ (k*(h0/k)^exp(-r*duration_months)), 
                   data=g,
                   start = list(k=k_s, h0=h0_s, r=r_s),
                   control = list(maxiter = 500))
  
  sum_gomp <- summary(fit.gomp)
  
  df$gomp_AIC[i] <- AIC(fit.gomp)
  #df$gomp_k[i] <- sum_gomp$parameters[1]
  #df$gomp_r[i] <- sum_gomp$parameters[3]
  #df$gomp_h0[i] <- sum_gomp$parameters[2]
  
  
  }, error=function(e){cat("ERROR :",conditionMessage(e), "\n")})
  
}
# Code generates some errors when the model doesn't fit for a particular 
# plot id, but this is expected.

################################################################################
# Selecting best model through AIC for treatments with 4+ observations

# select out AIC columns:
AIC <- c( "exp_AIC", "gomp_AIC", "lm_AIC")

#df_AIC <- select(df, AIC)
df_AIC <- df[,AIC]
# set the AIC for models that haven't fit to infinity

df_AIC[is.na(df_AIC)] <- Inf

# extract the value and the index of the lowest AIC:

df_AIC$model_AIC_val <- apply(df_AIC,1, FUN=min)
df_AIC$model <- max.col(-df_AIC[,1:3])

df_AIC$model_AIC_val <- df_AIC$model_AIC_val + 2
df_AIC$model_AIC <- NA


# check the best model fit +2 AIC compared to the other model types
# and sets the simplest model accordingly

for(i in 1:nrow(df_AIC)) {
  
  if(df_AIC$model_AIC_val[i]> df_AIC$gomp_AIC[i]){
    df_AIC$model_AIC[i] <- "gomp"
  }
  
  if(df_AIC$model_AIC_val[i]> df_AIC$exp_AIC[i]){
    df_AIC$model_AIC[i] <- "exp"
  }
  if(df_AIC$model_AIC_val[i]> df_AIC$lm_AIC[i]){
    df_AIC$model_AIC[i] <- "lm"
  }
}


df$model_AIC <- df_AIC$model_AIC

################################################################################
# Outputs for best model fits
################################################################################

df_h4_outputs <- df_h4[,c("plot_id", "duration_months", "height_cm_v2")]
df_h4_outputs$predict <- NA
df_h4_outputs$model <- NA
df_h4_outputs$code <- NA

df$code[df$plot_id == "GLL2013RFWA_CCDisF_1"]

# df_h4 contains all of the model parameter estimates for treatments with 4+ censuses
# df_h4_outputs is going to contain all of our outputs of the final models
# for treatments with 4+ censuses

for(i in 1:nrow(df)){
  tryCatch({
    
    print(i)
    
    # extract the treatment data from the full dataframe:
    p <- df$plot_id[i]
    m <- df$model_AIC[i]
    code <- df$code[i]
    g <- df_h4_outputs[df_h4_outputs$plot_id == p,]
    g <- g[,c("height_cm_v2", "duration_months")]
    h0 <- round(g$height_cm_v2[1], digits=2)
    
    if(m == "exp") {
      
      tmp.exp  <- getInitial(height_cm_v2~SS.exp(duration_months,h0, r), g)
      
      # add these variables into the dataframe: 
      r_s <- as.numeric(tmp.exp[2])
      h0_s <- as.numeric(tmp.exp[1])
      
      fit.exp <- nls(height_cm_v2 ~ h0* exp(r*duration_months),
                     start = list(h0=h0_s, r = r_s),
                     data = g,
                     control = list(maxiter = 500))
      
      sum_exp <- summary(fit.exp)
      
      r <- sum_exp$parameters[2]
      h0 <- sum_exp$parameters[1]
      
      #df$exp_r[i] <- r
      #df$exp_h0[i] <- h0
      
      # SGR:
      df$SGR_100[i] <- r
      df$SGR_200[i] <-  r
      df$SGR_300[i] <-  r
      
      df_h4_outputs$predict[df_h4_outputs$plot_id == p] <- predict(fit.exp)
      df_h4_outputs$model[df_h4_outputs$plot_id == p] <- "exp"
      
      # Time at reference height, AGR values, AGR se and SGR values
      vcov_e <- vcov(fit.exp) #variance covariance matrix
      
      df <- exp_AGR(fit.exp, df, 100, i, h0, r)
      df <- exp_AGR(fit.exp, df, 200, i, h0, r)
      df <- exp_AGR(fit.exp, df, 300, i, h0, r)
      
    }
    
    if(m =="lm"){
      
      mod_lm <- lm(height_cm_v2 ~ duration_months, data= g)
      
      h0 <- as.numeric(mod_lm$coefficients[1])
      r <- as.numeric(mod_lm$coefficients[2])
      
      df$lm_h0[i] <- h0
      df$lm_r[i] <- r
      
      # SGR
      
      df$SGR_100[i] <- df$lm_r[i] / 100
      df$SGR_200[i] <- df$lm_r[i] / 200
      df$SGR_300[i] <- df$lm_r[i] / 300
      
      df_h4_outputs$predict[df_h4_outputs$plot_id == p] <- predict(mod_lm)
      df_h4_outputs$model[df_h4_outputs$plot_id == p] <- "lm"
      
      # AGR:
      
      #variance covariance matrix:
      vcov_lm <- vcov(mod_lm)
      vcov_lm <- vcov_lm[-1,-1]
      
      df <- lm_AGR(mod_lm, df, 100, i, h0, r)
      df <- lm_AGR(mod_lm, df, 200, i, h0, r)
      df <- lm_AGR(mod_lm, df, 300, i, h0, r)
      
    }
    
    if (m=="gomp") {
      
      tmp.gomp <- getInitial(height_cm_v2 ~ SSgompertz(duration_months, Asym, b2, b3), data = g)
      fit.gomp_trial <- gnls(height_cm_v2 ~ SSgompertz(duration_months, Asym, b2, b3), data = g, weights= varExp(form = ~ fitted(.)))
      out.gomp <- output.gomp.gnls(fit.gomp_trial, g$duration_months)
      
      #summary(fit.gomp1)
      #vcov(fit.gomp1)
      
      k_s <- as.numeric(out.gomp$params[1])
      r_s <- as.numeric(out.gomp$params[2])
      h0_s <- as.numeric(out.gomp$params[3])
      
      fit.gomp <- nls(height_cm_v2 ~ (k*(h0/k)^exp(-r*duration_months)), 
                      data=g,
                      #algorithm = "port",
                      start = list(k=k_s, h0=h0_s, r=r_s),
                      #lower=list(0, (h0-10), 0),
                      control = list(maxiter = 500))
      
      
      summary_gomp <- summary(fit.gomp)
      
      vcov_g <- vcov(fit.gomp)
      
      k <- summary_gomp$parameters[1]
      h0 <- summary_gomp$parameters[2]
      r <- summary_gomp$parameters[3]
      
      df$gomp_k[i] <- k
      df$gomp_r[i] <- r
      df$gomp_h0[i] <- h0
      
      
      # SGR
      df$SGR_100[i] <- r*log(k/100)
      df$SGR_200[i] <- r*log(k/200)
      df$SGR_300[i] <- r*log(k/300)
      
      # AGR:
      df <- gompertz_AGR(fit.gomp, df, 100, i, h0, r, k)
      df <- gompertz_AGR(fit.gomp, df, 200, i, h0, r, k)
      df <- gompertz_AGR(fit.gomp, df,  300, i, h0, r, k)
      
      df_h4_outputs$predict[df_h4_outputs$plot_id == p] <- predict(fit.gomp)
      df_h4_outputs$model[df_h4_outputs$plot_id == p] <- "gomp"
      
    }
    df_h4_outputs$code[df_h4_outputs$plot_id == p] <- code
  }, error=function(e){cat("ERROR :",conditionMessage(e), "\n")})
  
}


################################################################################
# For sites with 3 census observations:
################################################################################

# subset out the data from the full dataset
df_h_nls <- df_h[!is.na(df_h$height_cm),]
df_h_nls <- df_h_nls[order(df_h_nls$duration_months),]
g <- as.data.frame(table(df_h_nls$plot_id))
g <- droplevels(g$Var1[g$Freq==3])
df_h3 <- df_h_nls[df_h_nls$plot_id %in% g,]
rm(g)

rm(df_h_nls)

id3 <- df_h3 %>% group_by(plot_id, UIC) %>% dplyr::summarise(length=n())

# create data frame for model outputs
df_3obs = data.frame(matrix(vector(), nrow(id3),26,
                       dimnames=list(c(), c("UIC", "plot_id","code", "no_census", "h_0",
                                            "exp_r","exp_h0", "exp_AIC",
                                            "lm_r", "lm_H0", "lm_AIC", 
                                             "SGR_100", "SGR_200", "SGR_300",
                                            "SGR100_var", "SGR200_var", "SGR300_var",
                                            "AGR_100", "AGR_200", "AGR_300",
                                            "AGR100_var", "AGR200_var", "AGR300_var",
                                            "t_100", "t_200", "t_300"))),
                stringsAsFactors=F)

df_3obs$plot_id <- id3$plot_id
df_3obs$UIC <- id3$UIC
df_3obs$no_census <- id3$length
df_3obs$code <- 1:nrow(df_3obs)
df_3obs <- left_join(df_3obs, df_refh[,c(1,4:6)], by="plot_id")

###################################################################################
### Fitting the linear model:

for(i in 1:nrow(df_3obs)){
  print(i)
  # extract the data from the full dataframe
  p <- df_3obs$plot_id[i]
  g <- df_h3[df_h3$plot_id == p,]
  g <- g[,c("height_cm_v2", "duration_months")]
  #set the starting height
  h_0 <- round(g$height_cm_v2[1], digits=2)
  df_3obs$h_0[i] <- h_0
  
  #Linear model:
  O <- lm(height_cm_v2 ~ duration_months, data= g)
  df_3obs$lm_H0[i] <- as.numeric(O$coefficients[1])
  df_3obs$lm_r[i] <- as.numeric(O$coefficients[2])
  df_3obs$lm_AIC[i] <- AIC(O)
}

##################################################################################
## Fitting the exponential model:
for(i in 1:nrow(df_3obs)){
  tryCatch({
    print(i)
    # extract the data from the full dataframe
    p <- df_3obs$plot_id[i]
    g <- df_h3[df_h3$plot_id == p,]
    g <- g[,c("height_cm_v2", "duration_months")]
    #set the starting height
    h_0 <- round(g$height_cm_v2[1], digits=2) 

    tmp.exp  <- getInitial(height_cm_v2~SS.exp(duration_months, h_0, r), g)
     
    # add these variables into the dataframe: 
    r_s <- as.numeric(tmp.exp[2])
    h0_s <- as.numeric(tmp.exp[1])
    
    fit.exp <- nls(height_cm_v2 ~ h0* exp(r*duration_months),
                   start = list(h0=h0_s, r = r_s),
                   data = g,
                   control = list(maxiter = 500))
    
    df_3obs$exp_AIC[i] <- AIC(fit.exp)
    
  }, error=function(e){cat("ERROR :",conditionMessage(e), "\n")})
}
#get warning:
#selfStart initializing functions should have a final '...' argument since R 4.1.0
# this doesn't affect outputs

# Selecting the best model based on AIC:

df_3obs$model_AIC <- max.col(-df_3obs[,c(8,11)], "first")
df_3obs$model_AIC[df_3obs$model_AIC ==1] <- "exp"
df_3obs$model_AIC[df_3obs$model_AIC ==2] <- "lm"

# two instances where the exponential model didn't fit:
df_3obs$exp_AIC[is.na(df_3obs$exp_AIC)] <- Inf

for(i in 1:nrow(df_3obs)){
  f <- df_3obs[i,]
  print(i)
  if(f$exp_AIC + 2 > f$lm_AIC){
    df_3obs$model_AIC[i] <- "lm"
}
}

# Create a subset dataframe for the outputs of the best model fit

df_h3_outputs <- df_h3[,c("plot_id", "duration_months", "height_cm_v2")]
df_h3_outputs$predict <- NA
df_h3_outputs$model <- NA
df_h3_outputs$code <- NA

# Refit the models for the best model fit, extract the AGR, SGR and AGR standard
# error for each reference height

for(i in 1:nrow(df_3obs)){
  tryCatch({
    print(i)
    
    # extract the data from the full dataframe
    p <- df_3obs$plot_id[i]
    m <- df_3obs$model[i]
    code <- df_3obs$code[i]
    g <- df_h3_outputs[df_h3_outputs$plot_id == p,]
    g <- g[,c("height_cm_v2", "duration_months")]
    h_0 <- round(g$height_cm_v2[1], digits=2)
    
    if(m == "exp") {
      tmp.exp  <- getInitial(height_cm_v2~SS.exp(duration_months,h_0, r), g)
      #fit.exp  <-  nls(height_cm_v2 ~ SS.exp(duration_months, h_0, r), data = g) 
      #out.exp  <- output.exp.nls(fit.exp, g$duration_months, CI=F)
      
      # add these variables into the dataframe: 
      r_s <- as.numeric(tmp.exp[2])
      h0_s <- as.numeric(tmp.exp[1])
      
      fit.exp <- nls(height_cm_v2 ~ h0* exp(r*duration_months),
                     start = list(h0=h0_s, r = r_s),
                     data = g,
                     control = list(maxiter = 500))
      
      sum_exp <- summary(fit.exp)
      
      r <- sum_exp$parameters[2]
      h0 <- sum_exp$parameters[1]
      
      df_3obs$exp_r[i] <- r
      df_3obs$exp_h0[i] <- h0
      
      # SGR:
      df_3obs$SGR_100[i] <- r
      df_3obs$SGR_200[i] <-  r
      df_3obs$SGR_300[i] <-  r
      
      df_h3_outputs$predict[df_h3_outputs$plot_id == p] <- predict(fit.exp)
      df_h3_outputs$model[df_h3_outputs$plot_id == p] <- "exp"
      
      #### range of AGR values:
      ## This process isn't really needed for the exponential model as the SGR is
      # R for all times but the calcuation is included here for completeness.
      
      # AGR standard errors:
      vcov_e <- vcov(fit.exp) #variance covariance matrix
      
      df_3obs <- exp_AGR(fit.exp, df_3obs, 100, i, h0, r)
      df_3obs <- exp_AGR(fit.exp, df_3obs, 200, i, h0, r)
      df_3obs <- exp_AGR(fit.exp, df_3obs, 300, i, h0, r)
      
    }
    
    if(m =="lm"){
      
      lm_mod <- lm(height_cm_v2 ~ duration_months, data= g)
      
      h0 <- as.numeric(lm_mod$coefficients[1])
      r <- as.numeric(lm_mod$coefficients[2])
      
      df_3obs$lm_h0[i] <- h0
      df_3obs$lm_r[i] <- r
      
      df_3obs$SGR_100[i] <- r / 100
      df_3obs$SGR_200[i] <- r / 200
      df_3obs$SGR_300[i] <- r / 300
      
      #SGR standard error
      vcov_lm <- vcov(lm_mod)
      vcov_lm <- vcov_lm[-1,-1]
      
      d_r100 <- 1/100
      d_r200 <- 1/200
      d_r300 <- 1/300
      
      df_3obs$SGR100_var[i] <- d_r100 %*% vcov_lm %*% d_r100
      df_3obs$SGR200_var[i] <- d_r200 %*% vcov_lm %*% d_r200
      df_3obs$SGR300_var[i] <- d_r300 %*% vcov_lm %*% d_r300
      #variance covariance matrix:
      
      ## AGR:
      
      df_3obs <- lm_AGR(mod_lm, df_3obs, 100, i, h0, r)
      df_3obs <- lm_AGR(mod_lm, df_3obs, 200, i, h0, r)
      df_3obs <- lm_AGR(mod_lm, df_3obs, 300, i, h0, r)
      
      df_h3_outputs$predict[df_h3_outputs$plot_id == p] <- predict(lm_mod)
      df_h3_outputs$model[df_h3_outputs$plot_id == p] <- "lm"
      
    }
    
  }, error=function(e){cat("ERROR :",conditionMessage(e), "\n")})
  # Will print the number it is on for each one so you can see if/when it gets stuck
}
warnings()

# get the warning about 
#  selfStart initializing functions should have a final '...' argument since R 4.1.0
# but this is just a warning not an error. 

##########################################################################
# FINAL DATA FRAMES:
# Creating the final dataframes for each reference size
##########################################################################

# subsetting dataframes
cols <- c( "UIC", "plot_id","code","no_census" ,"cm100","cm200","cm300",    
            "model_AIC", "SGR_100", "SGR_200", "SGR_300","AGR_100", "AGR_200", "AGR_300",
           "AGR100_var", "AGR200_var", "AGR300_var", "t_100", "t_200", "t_300")

df_h4_final <- df[, cols]
df_h3_final <- df_3obs[, cols]
names(df_3obs)

# combining data from treatments with 3 or 4+ censuses:
df_SGR <- rbind(df_h4_final, df_h3_final)

# removing SGR and AGR values in instances where the reference height 
# doesn't fall into the range of growth in the treatment:

df_SGR$SGR_100[is.na(df_SGR$cm100)] <- NA
df_SGR$SGR_200[is.na(df_SGR$cm200)] <- NA
df_SGR$SGR_300[is.na(df_SGR$cm300)] <- NA
df_SGR$AGR_100[is.na(df_SGR$cm100)] <- NA
df_SGR$AGR_200[is.na(df_SGR$cm200)] <- NA
df_SGR$AGR_300[is.na(df_SGR$cm300)] <- NA
df_SGR$AGR100_var[is.na(df_SGR$cm100)] <- NA
df_SGR$AGR200_var[is.na(df_SGR$cm200)] <- NA
df_SGR$AGR300_var[is.na(df_SGR$cm300)] <- NA
df_SGR$t_100[is.na(df_SGR$cm100)] <- NA
df_SGR$t_200[is.na(df_SGR$cm200)] <- NA
df_SGR$t_300[is.na(df_SGR$cm300)] <- NA
names(df_SGR)

### adding back in plot level information:

# environmental data: getting one row for each plot_id
df_h <- subset(df_h, select=-c(UIC))
# subset just one observation per plot id
df_h2 <-  unique(setDT(df_h)[order(duration_months)], by="plot_id")
df_h2 <- df_h2[,-"duration_months"]

# combining env data with SGR dataframes
df_SGR <- left_join(df_SGR, df_h2, by="plot_id")

df_100 <- subset(df_SGR, select = -c(SGR_200, SGR_300, AGR_200, AGR_300,
                                     t_200, t_300, AGR200_var, AGR300_var,
                                     cm200, cm300))
df_100 <- df_100[!is.na(df_100$cm100),]
colnames(df_100)[c(5,7,8,9,10)] <- c("ref_height", "SGR", "AGR", "AGR_var", "time")

write.csv(df_100, "df_ASGR100.csv")

df_200 <- subset(df_SGR, select = -c(SGR_100, SGR_300, AGR_100, AGR_300,
                                     t_100, t_300, AGR100_var, AGR300_var,
                                     cm100, cm300))
df_200 <- df_200[!is.na(df_200$cm200),]
colnames(df_200)[c(5,7,8,9,10)] <- c("ref_height", "SGR", "AGR", "AGR_var", "time")

write.csv(df_200, "df_ASGR200.csv")

df_300 <- subset(df_SGR, select = -c(SGR_100, SGR_200, AGR_200, AGR_100,
                                     t_200, t_100, AGR200_var, AGR100_var,
                                     cm200, cm100))
df_300 <- df_300[!is.na(df_300$cm300),]
colnames(df_100)[c(5,7,8,9,10)] <- c("ref_height", "SGR", "AGR", "AGR_var", "time")

write.csv(df_300, "df_ASGR300.csv")

#' The three data frames df_ASGR100, df_ASGR200 and df_ASGR300 produced from the script
#' 'height_ASGR_nlms.R' are the input files for the script 'script_SynthesisAnalysis_ASGR_MEMs.R' 

