######## FOR-RESTOR Synthesis Phil Trans Paper Analysis ##########
######## Banin, Raine et al                             ##########
######## 02 November 2023

###### Script for functions relating to calculating annual growth rates

###
# For each model (linear, exponential and gompertz), there is a function to:
#
# 1) Calculate the x value at a given y value from a model fit.
# This is used to estimate the time at which a reference height is reached.
#
# 2) Calculate the derivative of the original function with respect to each parameter.
# The number of these varies depending on the number of parameters in the model.
# This is used in the delta method for the calculation of the annual growth rate variance
#
# 3) Calculate the annual growth rate and annual growth rate variance
# This function combines functions 1) and 2) and is used in the code in `height_ASGR_nlms.R`.

###
# Standard naming: 

# h0 = height in cm at first census. 
# hcm = height in centimetres - for adding reference heights (in model as height_cm_v2)
# here as hcm for simplicity
# t = time (in the model this is entered duration_months, here as t for simplicity)

# For each model there is a function for :
# calculating x from y (). (in this case, t from hcm) This is the rearranged equation.


########################
# Linear:

#get x from y (get t from hcm):
lm_xfromy <- function(hcm, h0, r) (hcm - h0)/r

######################
# Exponential functions:

# get x from y (t from hcm):
exp_xfromy <- function(hcm, h0 ,r) (log(hcm/h0))/r

# derivative of equation with respect to h0:
dt_exp_h0 <- function(t, r) exp(r*t)

# derivative of equation with respect to r:
dt_exp_r <- function(t, h0, r) h0*t*exp(r*t)

#####################
# Gompertz functions:

# get x from y(t from hcm):
gomp_xfromy <- function(hcm, h0, r, k) -(1/r) * log(log(hcm/k) / log(h0/k))

# derivative of equation with respect to h0:
dt_gomp_h0 <- function(t, h0, r, k) (exp(-r*t)/h0) *(k*(h0/k)^(exp(-r*t)))

# derivative of equation with respect to r:
dt_gomp_r <- function(t, h0, r, k) -t*((log(h0)-log(k)) * exp(-r*t)) * k * ((h0/k)^exp(-r*t))

# derivative of equation with respect to k:
dt_gomp_k <- function (t, h0, r, k) exp(-r*t) * (exp(r*t) - 1) * (h0/k)^exp(-r*t)




#### Function for working out AGR:

# For exponential model fit:

exp_AGR <- function(fit.exp, df, ref_height, iRow, h0, r) {
  if(!is.na(df[, paste0("cm", ref_height)][iRow])) {
    
    t <- exp_xfromy(ref_height, h0, r)
    t_l <- t-6
    t_h <- t+6
    
    df[, paste0("t_", ref_height)][iRow] <- t
    df[, paste0("AGR_", ref_height)][iRow] <- 
      as.numeric(predict(fit.exp, newdata=list(duration_months=c(t_h))[1])) -
      as.numeric(predict(fit.exp, newdata=list(duration_months=c(t_l))[1]))
    
    # Calculating variance: calculate derivatives of variables for t_h - t_l
    
    # Derivative of h0 =e^rt
    d_h0 <- dt_exp_h0(t_h,r) - dt_exp_h0(t_l,r)
    
    # derivative of r = h_0*t*exp(r*t)
    d_r <- dt_exp_r(t_h, h0, r) - dt_exp_r(t_l, h0, r)
    
    ## add in to vector
    xx <- c(d_h0, d_r)
    
    # delta method to calculate variance:
    df[, paste0("AGR", ref_height, "_var")][iRow] <- as.numeric(xx %*% vcov_e %*% xx)
    
  }
  return(df)
}

# For gompertz model fits

gompertz_AGR <- function(fit.gompertz, df, ref_height, iRow, h0, r,k) {
  if(!is.na(df[, paste0("cm", ref_height)][iRow])) {
    
    t <-  gomp_xfromy(ref_height, h0, r, k)
    t_l <- t-6
    t_h <- t+6
    
    df[, paste0("AGR_", ref_height)][iRow] <-
      as.numeric(predict(fit.gomp, newdata=list(duration_months=c(t_h)))[1]) -
      as.numeric(predict(fit.gomp, newdata=list(duration_months=c(t_l)))[1])
    
    d_h0 <- dt_gomp_h0(t_h, h0, r, k) - dt_gomp_h0(t_l, h0, r, k)
    d_r <- dt_gomp_r(t_h, h0, r, k) -  dt_gomp_r(t_l, h0, r, k) 
    d_k <- dt_gomp_k(t_h, h0, r, k) - dt_gomp_k(t_l, h0, r, k)
    
    xx <- c(d_k, d_h0, d_r)
    
    df[, paste0("AGR", ref_height, "_var")][iRow] <- as.numeric(xx %*% vcov_g %*% xx)
    
    df[, paste0("t_", ref_height)][iRow]<- t
    
  }
  return(df)
}

# For linear model fits:

lm_AGR <- function(lm.fit,df, ref_height, iRow, h0, r){
  if(!is.na(df[, paste0("cm", ref_height)][iRow])) {
    
    t <- lm_xfromy(ref_height, h0, r)
    df[, paste0("t_", ref_height)][iRow] <- t
    df[, paste0("AGR_", ref_height)][iRow] <- r*12 #for 12 months   
    
    d_r <- (t+6) - (t-6) 
    df[, paste0("AGR", ref_height, "_var")][iRow]  <- as.numeric(d_r %*% vcov_lm %*% d_r)
    
  }
  return(df)
}



