######## FOR-RESTOR Synthesis Phil Trans Paper Analysis ##########
######## Authors: Lindsay F Banin, Elizabeth Raine ###############

######## PLANTED TREE - GROWTH ANALYSIS
library(Rcpp)
library(Matrix)
library(lme4)
library(nlme)
library(stats)
library(glmmTMB)
library(dplyr)
library(MuMIn)
library(performance)
library(DHARMa)
library(glmmTMB)
library(bbmle) ## for AICtab
library(ggplot2)
library(ggeffects)
library(emmeans)
library(cowplot)

###### Size-standardised growth rate (SGR) at 100, 200, 300cm
### Read in .csv files generated using script "height_ASGR_nlms.R"

ASGR100_dat <- read.csv("df_ASGR100_30Jun.csv")
ASGR200_dat <- read.csv("df_ASGR200_30Jun.csv")
ASGR300_dat <- read.csv("df_ASGR300_30Jun.csv")

### Select variables for analysis
asgr_modvars <- c("SGR","AGR", "AGR_var", "network_origin","study","UIC","plot_id","family","species_full","vals_elev","vals_temp","vals_maxTwm",
"vals_minTcm","vals_precip","vals_pwq","vals_pdq",
"comp_removal","shading","soil_prep","water_reg","fertilisation","protection","w_meanWD","forest_condition",
"planting_sp_no", "sample_weight","treatment")

ASGR100_dat <- ASGR100_dat[asgr_modvars]
ASGR200_dat <- ASGR200_dat[asgr_modvars]
ASGR300_dat <- ASGR300_dat[asgr_modvars]

###### ASGR 100: Run mixed models using lme4; Log transform AGR, centre & standardise variables
# Variance very low for UIC:species_full so this interaction was removed from the model formulation
M1_AGR100 <- lmer(log(AGR) ~ scale(w_meanWD)*network_origin + scale(vals_elev) + scale(vals_pdq) + scale(vals_temp) + scale(w_meanWD)*forest_condition + planting_sp_no  
+ (1|UIC) + (1|species_full),
data = ASGR100_dat,
na.action=na.fail,
REML=FALSE)
dr1_AGR100 <- dredge(M1_AGR100, rank = "AIC")
top_modsAGR1 <- subset(dr1_AGR100,delta <2)
top_modsAGR1
#importance(top_modsAGR1) [function now updated to 'sw']
sw(top_modsAGR1)

best_M1_AGR100 <- get.models(dr1_AGR100, 1)[[1]]
summary(best_M1_AGR100)
# Present best model without reference category and re-run as REML
best_M1_AGR100 <- lmer(log(AGR) ~ -1 + forest_condition + scale(w_meanWD):forest_condition
+ (1|UIC) + (1|species_full),
data = ASGR100_dat,
na.action=na.fail,
REML=TRUE)
summary(best_M1_AGR100)
ggpredict(best_M1_AGR100)
r2_nakagawa(best_M1_AGR100)
icc(best_M1_AGR100, by_group = TRUE) 

###### ASGR 200: Run mixed models using lme4; Log transform AGR, centre & standardise variables
# Variance very low for UIC:species_full so this interaction was removed from the model formulation
# Exclude one negative AGR
ASGR200.1_dat <- ASGR200_dat[!(ASGR200_dat$AGR < 0),]
M1_AGR200 <- lmer(log(AGR) ~ scale(w_meanWD)*network_origin + scale(vals_elev) + scale(vals_pdq) + scale(vals_temp) + scale(w_meanWD)*forest_condition + planting_sp_no  
+ (1|UIC) + (1|species_full),
data = ASGR200.1_dat,
na.action=na.fail,
REML=FALSE)
dr1_AGR200 <- dredge(M1_AGR200, rank = "AIC")
top_modsAGR2 <- subset(dr1_AGR200,delta <2)
top_modsAGR2
#importance(top_modsAGR2)[function now updated to 'sw']
sw(top_modsAGR2) 

best_M1_AGR200 <- get.models(dr1_AGR200, 1)[[1]]
summary(best_M1_AGR200)
# Present without reference category and re-run as REML
best_M1_AGR200 <- lmer(log(AGR) ~ -1 + forest_condition + scale(w_meanWD):forest_condition
+ (1|UIC) + (1|species_full),
data = ASGR200.1_dat,
na.action=na.fail,
REML=TRUE)
summary(best_M1_AGR200)
ggpredict(best_M1_AGR200)
r2_nakagawa(best_M1_AGR200)
icc(best_M1_AGR200, by_group = TRUE) # (1|UIC:species_full) var very low - remove

###### ASGR 300: Run mixed models using lme4; Log transform AGR, centre & standardise variables
# Variance very low for UIC:species_full and species_full so these random effects terms were removed from the model formulation
M1_AGR300 <- lmer(log(AGR) ~ scale(w_meanWD)*network_origin + scale(vals_elev) + scale(vals_pdq) + scale(vals_temp) + scale(w_meanWD)*forest_condition + planting_sp_no  
+ (1|UIC),
data = ASGR300_dat,
na.action=na.fail,
REML=FALSE)
dr1_AGR300 <- dredge(M1_AGR300, rank = "AIC")
top_modsAGR3 <- subset(dr1_AGR300,delta <2)
top_modsAGR3
importance(top_modsAGR3) 
best_M1_AGR300 <- get.models(dr1_AGR300, 1)[[1]]
summary(best_M1_AGR300)

best_M1_AGR300 <- lmer(log(AGR) ~ scale(w_meanWD)
+ (1|UIC),
data = ASGR300_dat,
na.action=na.fail,
REML=TRUE)
summary(best_M1_AGR300)
ggpredict(best_M1_AGR300)
r2_nakagawa(best_M1_AGR300)
icc(best_M1_AGR300, by_group = TRUE) # (1|UIC:species_full) and (1|species_full var very low - remove
ASGR300_dat$species_full <- as.factor(ASGR300_dat$species_full)
nlevels(ASGR300_dat$species_full)

###### PLOTTING
# Set colourblind palette for habitat classes - forest, open, plantation
cbp3 <- c("#0072B2", "#E69F00", "#009E73")

pmeWD_FC_100  <- ggemmeans(model = best_M1_AGR100, terms = c("w_meanWD[all]", "forest_condition[all]"))
preds1 <- as.data.frame(pmeWD_FC_100)
g100_WDFC <-
  ggplot() +
  geom_jitter(data =   ASGR100_dat %>% select(predicted = AGR, x = w_meanWD, group = forest_condition),
              aes(y = predicted, x = x,color=group), 
              alpha = .75, show.legend = TRUE) +
  scale_size_area() +
  geom_ribbon(data = preds1, 
              aes(x = x, y = predicted, ymax = conf.high, ymin = conf.low,
                  fill = group), alpha = .25) +
  geom_line(data = preds1, 
            aes(x = x, y = predicted, color = group,), size = .8) +
  scale_color_manual(values = cbp3, 
                     name = "Habitat condition", labels=c("Forest enrichment", "Open degraded", "Plantation enrichment"), aesthetics = c("color", "fill")) +
scale_x_continuous(limits = c(0.25,0.95), expand = c(0, 0)) +
scale_y_continuous(limits = c(0,750), expand = c(0, 0)) +                    
  theme_classic(base_size = 14) +
  theme(legend.position = c(0.5, 0.85),
        legend.background = element_blank()) +
  theme(axis.title = element_text(size = 14)) +
  labs(x = expression(paste("Wood density ", (g/cm^{3}))), y = expression(paste("Growth rate at 100 cm ", (cm/yr)))); g100_WDFC

pmeWD_FC_200  <- ggemmeans(model = best_M1_AGR200, terms = c("w_meanWD[all]", "forest_condition[all]"))
preds2 <- as.data.frame(pmeWD_FC_200)
g200_WDFC <-
  ggplot() +
  geom_jitter(data =   ASGR200.1_dat %>% select(predicted = AGR, x = w_meanWD, group = forest_condition),
              aes(y = predicted, x = x,color=group), 
              alpha = .75, show.legend = FALSE) +
  scale_size_area() +
  geom_ribbon(data = preds2, 
              aes(x = x, y = predicted, ymax = conf.high, ymin = conf.low,
                  fill = group), alpha = .25, show.legend = FALSE) +
  geom_line(data = preds2, 
            aes(x = x, y = predicted, color = group,), size = .8, show.legend = FALSE) +
  scale_color_manual(values = cbp3, 
                     name = "Forest condition", labels=c("Forest enrichment", "Open degraded", "Plantation enrichment"), aesthetics = c("color", "fill")) +
scale_x_continuous(limits = c(0.25,0.95), expand = c(0, 0)) +
scale_y_continuous(limits = c(0,750), expand = c(0, 0)) +   
  theme_classic(base_size = 14) +
  labs(x = expression(paste("Wood density ", (g/cm^{3}))), y = expression(paste("Growth rate at 200 cm ", (cm/yr)))); g200_WDFC

###
pmeWD_300  <- ggemmeans(model = best_M1_AGR300, terms = "w_meanWD[all]")
preds3 <- as.data.frame(pmeWD_300)
g300_WD <- 
  ggplot() + 
  geom_point(data =  ASGR300_dat %>% select(predicted = AGR, x = w_meanWD, group = forest_condition),
              aes(y = predicted, x = x,color=group), 
              alpha = .75, show.legend = FALSE) +
		scale_color_manual(values = cbp3, name = "Habitat condition", labels=c("Forest enrichment", "Open degraded", "Plantation enrichment"))+ 
  scale_size_area() +
  geom_ribbon(data = preds3, 
              aes(x = x, y = predicted, ymax = conf.high, ymin = conf.low),
              fill = "gray", alpha = .5) +
  geom_line(data = preds3, 
            aes(x = x, y = predicted), color = "black", size = .8) +
scale_x_continuous(limits = c(0.25,0.95), expand = c(0, 0)) +
scale_y_continuous(limits = c(0,750), expand = c(0, 0)) +   
  theme_classic(base_size = 14) +
  theme(axis.title = element_text(size = 14)) +
 labs(x = expression(paste("Wood density ", (g/cm^{3}))), y = expression(paste("Growth rate at 300 cm ", (cm/yr)))); g300_WD

## Compile plots
Growth_panel <- plot_grid(g100_WDFC, g200_WDFC, g300_WD, ncol=3, labels="auto",vjust = 1,hjust = 0.1)
Growth_panel
ggsave("growth_panel20Aug22_v1.png", plot=last_plot(), width=25, height=10, dpi=300, units="cm")

##### PLOTTING version 2 - truncate CI
pmeWD_FC_200  <- ggemmeans(model = best_M1_AGR200, terms = c("w_meanWD[all]", "forest_condition[all]"))
preds2 <- as.data.frame(pmeWD_FC_200)
preds2B <- preds2
preds2B$conf.high[preds2B$conf.high > 600] <- 600 
g200_WDFC <-
  ggplot() +
  geom_jitter(data =   ASGR200.1_dat %>% select(predicted = AGR, x = w_meanWD, group = forest_condition),
              aes(y = predicted, x = x,color=group), 
              alpha = .75, show.legend = FALSE) +
  scale_size_area() +
  geom_ribbon(data = preds2B, 
              aes(x = x, y = predicted, ymax = conf.high, ymin = conf.low,
                  fill = group), alpha = .25, show.legend = FALSE) +
  geom_line(data = preds2B, 
            aes(x = x, y = predicted, color = group,), size = .8, show.legend = FALSE) +
  scale_color_manual(values = cbp3, 
                     name = "Forest condition", labels=c("Forest enrichment", "Open degraded", "Plantation enrichment"), aesthetics = c("color", "fill")) +
scale_x_continuous(limits = c(0.25,0.95), expand = c(0, 0)) +
scale_y_continuous(limits = c(0,600), expand = c(0, 0)) +   
  theme_classic(base_size = 14) +
  labs(x = expression(paste("Wood density ", (g/cm^{3}))), y = expression(paste("Growth rate at 200 cm ", (cm/yr)))); g200_WDFC

##### See Supplementary Information, Banin, Raine et al. (2023) for in-depth explanation on simulation approach to
##### checking impact of uncertainty from non-linear fits on relationships between tree growth and covariates
##### Simulation code: AGR 100
# Data: logagrmean, logagrSE, for each of 'nobs' species-site combinations
logagrmean100 <- log(ASGR100_dat$AGR)
logagrSE100 <- sqrt(ASGR100_dat$AGR_var/(ASGR100_dat$AGR^2))
nobs <- 310 # number of unique plot_ids
nsim <- 500 ## specify number of simulated datasets to produce (1000 may not be realistic, depending how long models take to run)
nsim2 <- 1000 ## specify number of simulations to use for simulating parameter values within each dataset (this step is computationally fast, so can set this to a high number)
fits <- as.list(NULL)
for(k in 1:nsim){ ## loop over number of simulations
    logagrsim <- rnorm(nobs, logagrmean100, logagrSE100) ## generate each simulated dataset
	sim_dat100 <- data.frame(cbind(logagrsim,ASGR100_dat))
    fits[[k]] <- summary(lmer(logagrsim ~ -1 + forest_condition + scale(w_meanWD):forest_condition + (1|UIC) + (1|species_full), data = sim_dat100)) ## analyze each simulated dataset, in the same way the original data 'logsgrmean' were analyzed
}
## To generate simulated values for each parameter in the output:
ests1 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[1,1], fits$coef[1,2])}))
ests2 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[2,1], fits$coef[2,2])}))
ests3 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[3,1], fits$coef[3,2])})) 
ests4 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[4,1], fits$coef[4,2])}))
ests5 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[5,1], fits$coef[5,2])}))
ests6 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[6,1], fits$coef[6,2])}))
# Parameter estimates
mean(ests1)
mean(ests2)
mean(ests3)
mean(ests4)
mean(ests5)
mean(ests6)
# Parameter standard errors
sd(ests1)
sd(ests2)
sd(ests3)
sd(ests4)
sd(ests5)
sd(ests6)
## 95% CI for parameter 
quantile(ests1, c(0.025, 0.975))
quantile(ests2, c(0.025, 0.975))
quantile(ests3, c(0.025, 0.975)) 
quantile(ests4, c(0.025, 0.975))
quantile(ests5, c(0.025, 0.975))
quantile(ests6, c(0.025, 0.975))

### Simulation code: AGR 200
# Data: logagrmean, logagrSE, for each of 'nobs' species-site combinations
logagrmean200 <- log(ASGR200.1_dat$AGR)
logagrSE200 <- sqrt(ASGR200.1_dat$AGR_var/(ASGR200.1_dat$AGR^2))
nobs <- 341 # number of unique plot_ids
nsim <- 500 ## specify number of simulated datasets to produce (1000 may not be realistic, depending how long models take to run)
nsim2 <- 1000 ## specify number of simulations to use for simulating parameter values within each dataset (this step is computationally fast, so can set this to a high number)
fits <- as.list(NULL)
for(k in 1:nsim){ ## loop over number of simulations
    logagrsim200 <- rnorm(nobs, logagrmean200, logagrSE200) ## generate each simulated dataset
	sim_dat200 <- data.frame(cbind(logagrsim200,ASGR200.1_dat))
    fits[[k]] <- summary(lmer(logagrsim200 ~ -1 + forest_condition + scale(w_meanWD):forest_condition + (1|UIC) + (1|species_full), data = sim_dat200)) ## analyze each simulated dataset, in the same way the original data 'logsgrmean' were analyzed
}
## generate simulated values of paramete:
ests1 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[1,1], fits$coef[1,2])}))
ests2 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[2,1], fits$coef[2,2])}))
ests3 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[3,1], fits$coef[3,2])})) 
ests4 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[4,1], fits$coef[4,2])}))
ests5 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[5,1], fits$coef[5,2])}))
ests6 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[6,1], fits$coef[6,2])}))
# Parameter estimates
mean(ests1)
mean(ests2)
mean(ests3)
mean(ests4)
mean(ests5)
mean(ests6)
# Parameter standard errors
sd(ests1)
sd(ests2)
sd(ests3)
sd(ests4)
sd(ests5)
sd(ests6)
# 95% CI for parameter 
quantile(ests1, c(0.025, 0.975))
quantile(ests2, c(0.025, 0.975))
quantile(ests3, c(0.025, 0.975)) 
quantile(ests4, c(0.025, 0.975))
quantile(ests5, c(0.025, 0.975))
quantile(ests6, c(0.025, 0.975))

### Simulation code: AGR 300
# Data: logagrmean, logagrSE, for each of 'nobs' species-site combinations
logagrmean300 <- log(ASGR300_dat$AGR)
hist(logagrmean300)
logagrSE300 <- sqrt(ASGR300_dat$AGR_var/(ASGR300_dat$AGR^2))
hist(logagrSE300)
nobs <- 256 # number of unique plot_ids
nsim <- 500 ## specify number of simulated datasets to produce (1000 may not be realistic, depending how long models take to run)
nsim2 <- 1000 ## specify number of simulations to use for simulating parameter values within each dataset (this step is computationally fast, so can set this to a high number)
fits <- as.list(NULL)
for(k in 1:nsim){ ## loop over number of simulations
    logagrsim300 <- rnorm(nobs, logagrmean300, logagrSE300) ## generate each simulated dataset
	sim_dat300 <- data.frame(cbind(logagrsim300,ASGR300_dat))
    fits[[k]] <- summary(lmer(logagrsim300 ~ scale(w_meanWD) + (1|UIC), data = sim_dat300)) ## analyze each simulated dataset, in the same way the original data 'logsgrmean' were analyzed
}
## To generate simulated values for each parameter in the output:
ests1 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[1,1], fits$coef[1,2])}))
ests2 <- unlist(lapply(fits, function(fits){rnorm(nsim2, fits$coef[2,1], fits$coef[2,2])}))
mean(ests1)
mean(ests2)
sd(ests1)
sd(ests2)
quantile(ests1, c(0.025, 0.975))
quantile(ests2, c(0.025, 0.975))


####### Checking variation within/between times series to justify whether simulation needed
mean((logagrSE100)^2) # within time-series variation
var(logagrmean100) # between time-series variation
mean(logagrSE200^2) # within time-series variation
var(logagrmean200) # between time-series variation
mean((logagrSE300)^2) # average uncertainty within a time-series
var(logagrmean300) # variance in mean growth rates across sites/species


